# Copyright (C) 2023 NDP Systèmes (<http://www.ndp-systemes.fr>).
{
    "name": "Odoo Env to Config",
    "version": "1.0",
    "author": "NDP Systèmes",
    "maintainer": "NDP Systèmes",
    "category": "Module Client",
    "depends": ["base"],
    "description": """
Contient la cli commande pour Odoo pour creer une config depuis l'environment
=============================================================================
""",
    "website": "http://www.ndp-systemes.fr",
    "data": [],
    "demo": [],
    "test": [],
    "installable": False,
    "auto_install": False,
    "license": "AGPL-3",
    "application": False,
}
